"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.path2Curve = void 0;
const params_parser_1 = require("../parser/params-parser");
const clone_path_1 = require("../process/clone-path");
const fix_arc_1 = require("../process/fix-arc");
const normalize_path_1 = require("../process/normalize-path");
const is_curve_array_1 = require("../util/is-curve-array");
const segment_2_cubic_1 = require("../process/segment-2-cubic");
// import { fixPath } from '../process/fix-path';
function path2Curve(pathInput, needZCommandIndexes = false) {
    if ((0, is_curve_array_1.isCurveArray)(pathInput)) {
        const cloned = (0, clone_path_1.clonePath)(pathInput);
        if (needZCommandIndexes) {
            return [cloned, []];
        }
        else {
            return cloned;
        }
    }
    // fixPath will remove 'Z' command
    // const path = fixPath(normalizePath(pathInput));
    const path = (0, normalize_path_1.normalizePath)(pathInput);
    const params = Object.assign({}, params_parser_1.paramsParser);
    const allPathCommands = [];
    let pathCommand = '';
    let ii = path.length;
    let segment;
    let seglen;
    const zCommandIndexes = [];
    for (let i = 0; i < ii; i += 1) {
        if (path[i])
            [pathCommand] = path[i];
        allPathCommands[i] = pathCommand;
        const curveSegment = (0, segment_2_cubic_1.segmentToCubic)(path[i], params);
        path[i] = curveSegment;
        (0, fix_arc_1.fixArc)(path, allPathCommands, i);
        ii = path.length; // solves curveArrays ending in Z
        // keep Z command account for lineJoin
        // @see https://github.com/antvis/util/issues/68
        if (pathCommand === 'Z') {
            zCommandIndexes.push(i);
        }
        segment = path[i];
        seglen = segment.length;
        params.x1 = +segment[seglen - 2];
        params.y1 = +segment[seglen - 1];
        params.x2 = +segment[seglen - 4] || params.x1;
        params.y2 = +segment[seglen - 3] || params.y1;
    }
    // validate
    if (needZCommandIndexes) {
        return [path, zCommandIndexes];
    }
    else {
        return path;
    }
}
exports.path2Curve = path2Curve;
//# sourceMappingURL=path-2-curve.js.map