"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizePath = void 0;
const is_normalized_array_1 = require("../util/is-normalized-array");
const params_parser_1 = require("../parser/params-parser");
const path_2_absolute_1 = require("../convert/path-2-absolute");
const clone_path_1 = require("./clone-path");
const normalize_segment_1 = require("./normalize-segment");
/**
 * @example
 * const path = 'M0 0 H50';
 * const normalizedPath = SVGPathCommander.normalizePath(path);
 * // result => [['M', 0, 0], ['L', 50, 0]]
 */
function normalizePath(pathInput) {
    if ((0, is_normalized_array_1.isNormalizedArray)(pathInput)) {
        return (0, clone_path_1.clonePath)(pathInput);
    }
    const path = (0, path_2_absolute_1.path2Absolute)(pathInput);
    const params = Object.assign({}, params_parser_1.paramsParser);
    const allPathCommands = [];
    const ii = path.length;
    let pathCommand = '';
    for (let i = 0; i < ii; i += 1) {
        [pathCommand] = path[i];
        // Save current path command
        allPathCommands[i] = pathCommand;
        path[i] = (0, normalize_segment_1.normalizeSegment)(path[i], params);
        const segment = path[i];
        const seglen = segment.length;
        params.x1 = +segment[seglen - 2];
        params.y1 = +segment[seglen - 1];
        params.x2 = +segment[seglen - 4] || params.x1;
        params.y2 = +segment[seglen - 3] || params.y1;
    }
    return path;
}
exports.normalizePath = normalizePath;
//# sourceMappingURL=normalize-path.js.map