"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.segmentToCubic = void 0;
const arc_2_cubic_1 = require("./arc-2-cubic");
const quad_2_cubic_1 = require("./quad-2-cubic");
const line_2_cubic_1 = require("./line-2-cubic");
function segmentToCubic(segment, params) {
    const [pathCommand] = segment;
    const values = segment.slice(1).map(Number);
    const [x, y] = values;
    let args;
    const { x1: px1, y1: py1, x: px, y: py } = params;
    if (!'TQ'.includes(pathCommand)) {
        params.qx = null;
        params.qy = null;
    }
    switch (pathCommand) {
        case 'M':
            params.x = x;
            params.y = y;
            return segment;
        case 'A':
            args = [px1, py1, ...values];
            // @ts-ignore
            return ['C', ...(0, arc_2_cubic_1.arcToCubic)(...args)];
        case 'Q':
            params.qx = x;
            params.qy = y;
            args = [px1, py1, ...values];
            // @ts-ignore
            return ['C', ...(0, quad_2_cubic_1.quadToCubic)(...args)];
        case 'L':
            return ['C', ...(0, line_2_cubic_1.lineToCubic)(px1, py1, x, y)];
        case 'Z':
            // prevent NaN from divide 0
            if (px1 === px && py1 === py) {
                return ['C', px1, py1, px, py, px, py];
            }
            return ['C', ...(0, line_2_cubic_1.lineToCubic)(px1, py1, px, py)];
        default:
    }
    return segment;
}
exports.segmentToCubic = segmentToCubic;
//# sourceMappingURL=segment-2-cubic.js.map