"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathBBoxTotalLength = void 0;
const path_length_factory_1 = require("./path-length-factory");
/**
 * Returns the bounding box of a shape.
 */
function getPathBBoxTotalLength(path) {
    if (!path) {
        return {
            length: 0,
            x: 0,
            y: 0,
            width: 0,
            height: 0,
            x2: 0,
            y2: 0,
            cx: 0,
            cy: 0,
            cz: 0,
        };
    }
    const { length, min: { x: xMin, y: yMin }, max: { x: xMax, y: yMax }, } = (0, path_length_factory_1.pathLengthFactory)(path);
    const width = xMax - xMin;
    const height = yMax - yMin;
    return {
        length,
        width,
        height,
        x: xMin,
        y: yMin,
        x2: xMax,
        y2: yMax,
        cx: xMin + width / 2,
        cy: yMin + height / 2,
        // an estimted guess
        cz: Math.max(width, height) + Math.min(width, height) / 2,
    };
}
exports.getPathBBoxTotalLength = getPathBBoxTotalLength;
//# sourceMappingURL=get-path-bbox-total-length.js.map