"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPropertiesAtLength = void 0;
const parse_path_string_1 = require("../parser/parse-path-string");
const get_total_length_1 = require("./get-total-length");
/**
 * Returns the segment, its index and length as well as
 * the length to that segment at a given length in a path.
 */
function getPropertiesAtLength(pathInput, distance) {
    const pathArray = (0, parse_path_string_1.parsePathString)(pathInput);
    if (typeof pathArray === 'string') {
        throw TypeError(pathArray);
    }
    let pathTemp = [...pathArray];
    let pathLength = (0, get_total_length_1.getTotalLength)(pathTemp);
    let index = pathTemp.length - 1;
    let lengthAtSegment = 0;
    let length = 0;
    let segment = pathArray[0];
    const [x, y] = segment.slice(-2);
    const point = { x, y };
    // If the path is empty, return 0.
    if (index <= 0 || !distance || !Number.isFinite(distance)) {
        return {
            segment,
            index: 0,
            length,
            point,
            lengthAtSegment,
        };
    }
    if (distance >= pathLength) {
        pathTemp = pathArray.slice(0, -1);
        lengthAtSegment = (0, get_total_length_1.getTotalLength)(pathTemp);
        length = pathLength - lengthAtSegment;
        return {
            segment: pathArray[index],
            index,
            length,
            lengthAtSegment,
        };
    }
    const segments = [];
    while (index > 0) {
        segment = pathTemp[index];
        pathTemp = pathTemp.slice(0, -1);
        lengthAtSegment = (0, get_total_length_1.getTotalLength)(pathTemp);
        length = pathLength - lengthAtSegment;
        pathLength = lengthAtSegment;
        segments.push({
            segment,
            index,
            length,
            lengthAtSegment,
        });
        index -= 1;
    }
    return segments.find(({ lengthAtSegment: l }) => l <= distance);
}
exports.getPropertiesAtLength = getPropertiesAtLength;
//# sourceMappingURL=get-properties-at-length.js.map