"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPropertiesAtPoint = void 0;
const parse_path_string_1 = require("../parser/parse-path-string");
const normalize_path_1 = require("../process/normalize-path");
const get_point_at_length_1 = require("./get-point-at-length");
const get_properties_at_length_1 = require("./get-properties-at-length");
const get_total_length_1 = require("./get-total-length");
/**
 * Returns the point and segment in path closest to a given point as well as
 * the distance to the path stroke.
 * @see https://bl.ocks.org/mbostock/8027637
 */
function getPropertiesAtPoint(pathInput, point) {
    const path = (0, parse_path_string_1.parsePathString)(pathInput);
    const normalPath = (0, normalize_path_1.normalizePath)(path);
    const pathLength = (0, get_total_length_1.getTotalLength)(path);
    const distanceTo = (p) => {
        const dx = p.x - point.x;
        const dy = p.y - point.y;
        return dx * dx + dy * dy;
    };
    let precision = 8;
    let scan;
    let scanDistance = 0;
    let closest;
    let bestLength = 0;
    let bestDistance = Infinity;
    // linear scan for coarse approximation
    for (let scanLength = 0; scanLength <= pathLength; scanLength += precision) {
        scan = (0, get_point_at_length_1.getPointAtLength)(normalPath, scanLength);
        scanDistance = distanceTo(scan);
        if (scanDistance < bestDistance) {
            closest = scan;
            bestLength = scanLength;
            bestDistance = scanDistance;
        }
    }
    // binary search for precise estimate
    precision /= 2;
    let before;
    let after;
    let beforeLength = 0;
    let afterLength = 0;
    let beforeDistance = 0;
    let afterDistance = 0;
    while (precision > 0.5) {
        beforeLength = bestLength - precision;
        before = (0, get_point_at_length_1.getPointAtLength)(normalPath, beforeLength);
        beforeDistance = distanceTo(before);
        afterLength = bestLength + precision;
        after = (0, get_point_at_length_1.getPointAtLength)(normalPath, afterLength);
        afterDistance = distanceTo(after);
        if (beforeLength >= 0 && beforeDistance < bestDistance) {
            closest = before;
            bestLength = beforeLength;
            bestDistance = beforeDistance;
        }
        else if (afterLength <= pathLength && afterDistance < bestDistance) {
            closest = after;
            bestLength = afterLength;
            bestDistance = afterDistance;
        }
        else {
            precision /= 2;
        }
    }
    const segment = (0, get_properties_at_length_1.getPropertiesAtLength)(path, bestLength);
    const distance = Math.sqrt(bestDistance);
    return { closest, distance, segment };
}
exports.getPropertiesAtPoint = getPropertiesAtPoint;
//# sourceMappingURL=get-properties-at-point.js.map